@@ -3,17 +3,17 @@ use crate::token::{Token, TokenKind};
 use crate::Config;
 use crate::NcclError;
 
-pub(crate) const TOP_LEVEL_KEY: &'static str = "__top_level__";
+pub(crate) const TOP_LEVEL_KEY: &str = "__top_level__";
 
 #[derive(Clone, Copy)]
 enum Indent {
     TopLevel,
-    Tabs { level: u8 },
-    Spaces { width: u8, level: u8 },
+    Tabs { level: usize },
+    Spaces { width: usize, level: usize },
 }
 
 impl Indent {
-    fn level_tabs(&self) -> u8 {
+    fn level_tabs(&self) -> usize {
         match self {
             Indent::TopLevel => 0,
             &Indent::Tabs { level } => level,
@@ -21,7 +21,7 @@ impl Indent {
         }
     }
 
-    fn level_spaces(&self) -> u8 {
+    fn level_spaces(&self) -> usize {
         match self {
             Indent::TopLevel => 0,
             Indent::Tabs { .. } => unreachable!(),
@@ -29,7 +29,7 @@ impl Indent {
         }
     }
 
-    fn width(&self) -> Option<u8> {
+    fn width(&self) -> Option<usize> {
         match self {
             Indent::TopLevel => None,
             Indent::Tabs { .. } => unreachable!(),
@@ -48,7 +48,7 @@ impl Indent {
         }
     }
 
-    fn increase_spaces(&self, width: u8) -> Indent {
+    fn increase_spaces(&self, width: usize) -> Indent {
         match self {
             Indent::TopLevel => Indent::Spaces { width, level: 1 },
             Indent::Tabs { level } => Indent::Tabs { level: level + 1 },
@@ -58,10 +58,18 @@ impl Indent {
             },
         }
     }
+
+    fn is_tabs_or_top_level(&self) -> bool {
+        matches!(self, Indent::Tabs { .. }) || matches!(self, Indent::TopLevel)
+    }
+
+    fn is_spaces_or_top_level(&self) -> bool {
+        matches!(self, Indent::Spaces { .. }) || matches!(self, Indent::TopLevel)
+    }
 }
 
 pub(crate) fn parse<'a>(scanner: &mut Scanner<'a>) -> Result<Config<'a>, NcclError> {
-    parse_with(scanner, &Config::new(TOP_LEVEL_KEY))
+    parse_with(scanner, &Config::new(TOP_LEVEL_KEY, false))
 }
 
 pub(crate) fn parse_with<'a>(
@@ -70,7 +78,7 @@ pub(crate) fn parse_with<'a>(
 ) -> Result<Config<'a>, NcclError> {
     let mut config = original.clone();
 
-    while scanner.peek_token(0)?.kind != TokenKind::EOF {
+    while scanner.peek_token(0)?.kind != TokenKind::Eof {
         parse_kv(scanner, Indent::TopLevel, &mut config)?;
     }
 
@@ -82,17 +90,17 @@ fn parse_kv<'a>(
     indent: Indent,
     parent: &mut Config<'a>,
 ) -> Result<(), NcclError> {
-    let value = consume(scanner, TokenKind::Value)?.lexeme;
+    let value = consume_value(scanner)?;
     let mut node = {
-        if parent.has_value(value) {
-            parent[value].clone()
+        if parent.has_value(value.lexeme) {
+            parent[value.lexeme].clone()
         } else {
-            Config::new(value)
+            Config::new(value.lexeme, value.kind == TokenKind::QuotedValue)
         }
     };
 
     match scanner.peek_token(0)?.kind {
-        TokenKind::Tabs(tabs) => {
+        TokenKind::Tabs(tabs) if indent.is_tabs_or_top_level() => {
             let next_indent = indent.increase_tabs();
             if tabs == next_indent.level_tabs() {
                 while scanner.peek_token(0)?.kind == TokenKind::Tabs(next_indent.level_tabs()) {
@@ -102,7 +110,8 @@ fn parse_kv<'a>(
             }
         }
 
-        TokenKind::Spaces(spaces) if matches!(indent, Indent::Spaces { .. } | Indent::TopLevel) => {
+        //TokenKind::Spaces(spaces) if matches!(indent, Indent::Spaces { .. } | Indent::TopLevel) => {
+        TokenKind::Spaces(spaces) if indent.is_spaces_or_top_level() => {
             let next_indent = indent.increase_spaces(indent.width().unwrap_or(spaces));
             if spaces == next_indent.level_spaces() {
                 while scanner.peek_token(0)?.kind == TokenKind::Spaces(next_indent.level_spaces()) {
@@ -119,6 +128,18 @@ fn parse_kv<'a>(
     Ok(())
 }
 
+fn consume_value<'a>(scanner: &mut Scanner<'a>) -> Result<Token<'a>, NcclError> {
+    let tok = scanner.next_token()?;
+    match tok.kind {
+        TokenKind::Value | TokenKind::QuotedValue => Ok(tok),
+        _ => Err(NcclError::UnexpectedToken {
+            span: tok.span,
+            expected: TokenKind::Value,
+            got: tok.kind,
+        }),
+    }
+}
+
 fn consume<'a>(scanner: &mut Scanner<'a>, kind: TokenKind) -> Result<Token<'a>, NcclError> {
     let tok = scanner.next_token()?;
     if tok.kind == kind {
@@ -157,28 +178,35 @@ mod test {
         assert_eq!(
             config,
             Config {
+                quoted: false,
                 key: TOP_LEVEL_KEY,
                 value: map![
                     "jackson" => Config {
+                        quoted: false,
                         key: "jackson",
                         value: map![
                             "easy" => Config {
+                                quoted: false,
                                 key: "easy",
                                 value: map![
                                     "abc" => Config {
+                                        quoted: false,
                                         key: "abc",
                                         value: map![]
                                     },
                                     "123" => Config {
+                                        quoted: false,
                                         key: "123",
                                         value: map![]
                                     }
                                 ]
                             },
                             "hopefully" => Config {
+                                quoted: false,
                                 key: "hopefully",
                                 value: map![
                                     "tabs work" => Config {
+                                        quoted: false,
                                         key: "tabs work",
                                         value: map![]
                                     }
@@ -199,41 +227,51 @@ mod test {
         assert_eq!(
             config,
             Config {
+                quoted: false,
                 key: TOP_LEVEL_KEY,
                 value: map![
                     "server" => Config {
+                        quoted: false,
                         key: "server",
                         value: map![
                             "domain" => Config {
+                                quoted: false,
                                 key: "domain",
                                 value: map![
                                     "example.com" => Config {
+                                        quoted: false,
                                         key: "example.com",
                                         value: map![]
                                     },
                                     "www.example.com" => Config {
+                                        quoted: false,
                                         key: "www.example.com",
                                         value: map![]
                                     }
                                 ]
                             },
                             "port" => Config {
+                                quoted: false,
                                 key: "port",
                                 value: map![
                                     "80" => Config {
+                                        quoted: false,
                                         key: "80",
                                         value: map![]
                                     },
                                     "443" => Config {
+                                        quoted: false,
                                         key: "443",
                                         value: map![]
                                     }
                                 ]
                             },
                             "root" => Config {
+                                quoted: false,
                                 key: "root",
                                 value: map![
                                     "/var/www/html" => Config {
+                                        quoted: false,
                                         key: "/var/www/html",
                                         value: map![]
                                     }
