@@ -1,9 +1,10 @@
 #[derive(Debug, PartialEq, Clone, Copy)]
 pub enum TokenKind {
     Value,
-    Tabs(u8),
-    Spaces(u8),
-    EOF,
+    QuotedValue,
+    Tabs(usize),
+    Spaces(usize),
+    Eof,
 }
 
 #[derive(Debug, PartialEq, Clone)]
